@@ -1,8 +1,10 @@
 
 #' Orthogonalizing EM
 #'
-#' @param x input matrix (sparse matrices not yet implemented). 
-#' Each row is an observation, each column corresponds to a covariate
+#' @param x input matrix of dimension n x p (sparse matrices not yet implemented). 
+#' Each row is an observation, each column corresponds to a covariate. The xval.oem() function
+#' is optimized for n >> p settings and may be very slow when p > n, so please use other packages
+#' such as \code{glmnet}, \code{ncvreg}, \code{grpreg}, or \code{gglasso} when p > n or p approx n.
 #' @param y numeric response vector of length \code{nobs = nrow(x)}.
 #' @param nfolds integer number of cross validation folds. 3 is the minimum number allowed. defaults to 10
 #' @param foldid an optional vector of values between 1 and \code{nfold} specifying which fold each observation belongs to.
@@ -13,8 +15,27 @@
 #' they measure the deviation from the fitted mean to the response.
 #' @param ncores Integer scalar that specifies the number of threads to be used
 #' @param family \code{"gaussian"} for least squares problems, \code{"binomial"} for binary response (not implemented yet). 
-#' @param penalty Specification of penalty type in lowercase letters. Choices include \code{"lasso"}, 
-#' \code{"ols"} (Ordinary least squares, no penaly), \code{"elastic.net"}, \code{"scad"}, \code{"mcp"}, \code{"grp.lasso"}
+#' @param penalty Specification of penalty type. Choices include:
+#' \itemize{
+#'    \item{\code{"elastic.net"}}{ - elastic net penalty, extra parameters: \code{"alpha"}}
+#'    \item{\code{"lasso"}}{ - lasso penalty}
+#'    \item{\code{"ols"}}{ - ordinary least squares}
+#'    \item{\code{"mcp"}}{ - minimax concave penalty, extra parameters: \code{"gamma"}}
+#'    \item{\code{"scad"}}{ - smoothly clipped absolute deviation, extra parameters: \code{"gamma"}}
+#'    \item{\code{"mcp.net"}}{ - minimax concave penalty + l2 penalty, extra parameters: 
+#'    \code{"gamma"}, \code{"alpha"}}
+#'    \item{\code{"scad.net"}}{ - smoothly clipped absolute deviation + l2 penalty, extra parameters: 
+#'    \code{"gamma"}, \code{"alpha"}}
+#'    \item{\code{"grp.lasso"}}{ - group lasso penalty}
+#'    \item{\code{"grp.lasso.net"}}{ - group lasso penalty + l2 penalty, extra parameters: \code{"alpha"}}
+#'    \item{\code{"grp.mcp"}}{ - group minimax concave penalty, extra parameters: \code{"gamma"}}
+#'    \item{\code{"grp.scad"}}{ - group smoothly clipped absolute deviation, extra parameters: \code{"gamma"}}
+#'    \item{\code{"grp.mcp.net"}}{ - group minimax concave penalty + l2 penalty, extra parameters: \code{"gamma"}, \code{"alpha"}}
+#'    \item{\code{"grp.scad.net"}}{ - group smoothly clipped absolute deviation + l2 penalty, extra parameters: \code{"gamma"}, \code{"alpha"}}
+#'    \item{\code{"sparse.grp.lasso"}}{ - sparse group lasso penalty (group lasso + lasso), extra parameters: \code{"tau"}}
+#' }
+#' Careful consideration is required for the group lasso, group MCP, and group SCAD penalties. Groups as specified by the \code{groups} argument 
+#' should be chosen in a sensible manner.
 #' @param weights observation weights. defaults to 1 for each observation (setting weight vector to 
 #' length 0 will default all weights to 1)
 #' @param lambda A user supplied lambda sequence. By default, the program computes
@@ -25,8 +46,10 @@
 #' value (i.e. the smallest value for which all coefficients are zero). The default
 #' depends on the sample size nobs relative to the number of variables nvars. If
 #' \code{nobs > nvars}, the default is 0.0001, close to zero. 
-#' @param alpha mixing value for elastic.net. penalty applied is (1 - alpha) * (ridge penalty) + alpha * (lasso penalty)
+#' @param alpha mixing value for \code{elastic.net}, \code{mcp.net}, \code{scad.net}, \code{grp.mcp.net}, \code{grp.scad.net}. 
+#' penalty applied is ((1 - alpha)/alpha) * (ridge penalty) + (lasso/mcp/mcp/grp.lasso penalty)
 #' @param gamma tuning parameter for SCAD and MCP penalties. must be >= 1
+#' @param tau mixing value for \code{sparse.grp.lasso}. penalty applied is (1 - tau) * (group lasso penalty) + tau * (lasso penalty)
 #' @param groups A vector of describing the grouping of the coefficients. See the example below. All unpenalized variables
 #' should be put in group 0
 #' @param penalty.factor Separate penalty factors can be applied to each coefficient. 
@@ -69,6 +92,15 @@
 #' system.time(xfit <- xval.oem(x = x, y = y, 
 #'                              penalty = c("lasso", "grp.lasso"), 
 #'                              groups = rep(1:20, each = 5)))
+#'                              
+#' system.time(xfit2 <- xval.oem(x = x, y = y, 
+#'                               penalty = c("lasso", "grp.lasso",
+#'                                           "mcp",       "scad", 
+#'                                           "mcp.net",   "scad.net",
+#'                                           "grp.lasso", "grp.lasso.net",
+#'                                           "grp.mcp",   "grp.scad",
+#'                                           "sparse.grp.lasso"), 
+#'                               groups = rep(1:20, each = 5)))
 #' 
 xval.oem <- function(x, 
                      y, 
@@ -77,13 +109,22 @@ xval.oem <- function(x,
                      type.measure     = c("mse", "deviance", "class", "auc", "mae"),
                      ncores           = -1,
                      family           = c("gaussian", "binomial"),
-                     penalty          = c("elastic.net", "lasso", "ols", "mcp", "scad", "grp.lasso"),
+                     penalty          = c("elastic.net", 
+                                          "lasso", 
+                                          "ols", 
+                                          "mcp",           "scad", 
+                                          "mcp.net",       "scad.net",
+                                          "grp.lasso",     "grp.lasso.net",
+                                          "grp.mcp",       "grp.scad",
+                                          "grp.mcp.net",   "grp.scad.net",
+                                          "sparse.grp.lasso"),
                      weights          = numeric(0),
                      lambda           = numeric(0),
                      nlambda          = 100L,
                      lambda.min.ratio = NULL,
                      alpha            = 1,
                      gamma            = 3,
+                     tau              = 0.5,
                      groups           = numeric(0),
                      penalty.factor   = NULL,
                      group.weights    = NULL,
@@ -95,8 +136,20 @@ xval.oem <- function(x,
                      irls.tol         = 1e-3,
                      compute.loss     = FALSE) 
 {
-    family  <- match.arg(family)
-    penalty <- match.arg(penalty, several.ok = TRUE)
+    this.call    <- match.call()
+    
+    family       <- match.arg(family)
+    
+    ## don't default to fitting all penalties!
+    ## only allow multiple penalties if the user
+    ## explicitly chooses multiple penalties
+    if ("penalty" %in% names(this.call))
+    {
+        penalty  <- match.arg(penalty, several.ok = TRUE)
+    } else 
+    {
+        penalty  <- match.arg(penalty, several.ok = FALSE)
+    }
     
     if (missing(type.measure)) 
         type.measure = "default"
@@ -114,7 +167,8 @@ xval.oem <- function(x,
     if (p >= n)
     {
         stop("number of observations must be greater than the number of variables
-             for xval, use cv.oem instead.")
+             for xval, use cv.oem instead, or, preferably, use another package such as
+             glmnet for the lasso, ncvreg for MCP/SCAD, or grpreg or gglasso for group lasso.")
     }
     
     if (is.null(foldid)) 
@@ -163,7 +217,7 @@ xval.oem <- function(x,
     }
     penalty.factor <- drop(penalty.factor)
     
-    if (any(penalty == "grp.lasso")) {
+    if (any(grep("grp", penalty) > 0)) {
         if (length(groups) != p) {
             stop("groups must have same length as number of columns in x")
         }
@@ -255,6 +309,7 @@ xval.oem <- function(x,
     nlambda       <- as.integer(nlambda)
     alpha         <- as.double(alpha[1])
     gamma         <- as.double(gamma[1])
+    tau           <- as.double(tau[1])
     tol           <- as.double(tol)
     irls.tol      <- as.double(irls.tol)
     irls.maxit    <- as.integer(irls.maxit)
@@ -297,6 +352,7 @@ xval.oem <- function(x,
                                                     lambda.min.ratio,
                                                     alpha,
                                                     gamma,
+                                                    tau,
                                                     penalty.factor,
                                                     standardize,
                                                     intercept,
@@ -318,6 +374,7 @@ xval.oem <- function(x,
                                                     lambda.min.ratio,
                                                     alpha,
                                                     gamma,
+                                                    tau,
                                                     penalty.factor,
                                                     standardize,
                                                     intercept,
@@ -332,8 +389,10 @@ xval.oem <- function(x,
         rownames(res$beta[[i]]) <- c("(Intercept)", varnames)
     }
     
+    names(res$beta) <- penalty
+    
     nz <- lapply(1:length(res$beta), function(m) 
-        sapply(predict.oem(res, type = "nonzero", which.model = m), length) - 1
+        sapply(predict.oem(res, type = "nonzero", which.model = m), length) 
     )
     
     lamin        <- getmin(res$lambda, res$cvm, res$cvsd)
@@ -373,6 +432,7 @@ oemfit_xval.gaussian <- function(is.sparse,
                                  lambda.min.ratio,
                                  alpha,
                                  gamma,
+                                 tau,
                                  penalty.factor,
                                  standardize,
                                  intercept,
@@ -405,6 +465,7 @@ oemfit_xval.gaussian <- function(is.sparse,
                      lambda.min.ratio,
                      alpha,
                      gamma,
+                     tau,
                      penalty.factor,
                      standardize,
                      intercept,
@@ -429,6 +490,7 @@ oemfit_xval.gaussian <- function(is.sparse,
                      lambda.min.ratio,
                      alpha,
                      gamma,
+                     tau,
                      penalty.factor,
                      standardize,
                      intercept,
@@ -462,6 +524,7 @@ oemfit_xval.binomial <- function(is.sparse,
                                  lambda.min.ratio,
                                  alpha,
                                  gamma,
+                                 tau,
                                  penalty.factor,
                                  standardize,
                                  intercept,
@@ -496,6 +559,7 @@ oemfit_xval.binomial <- function(is.sparse,
                      lambda.min.ratio,
                      alpha,
                      gamma,
+                     tau,
                      penalty.factor,
                      standardize,
                      intercept,
@@ -520,6 +584,7 @@ oemfit_xval.binomial <- function(is.sparse,
                      lambda.min.ratio,
                      alpha,
                      gamma,
+                     tau,
                      penalty.factor,
                      standardize,
                      intercept,
