@@ -59,8 +59,6 @@ protected:
     MatrixXd XX;                // X'X
     MatrixXd A;                 // A = d * I - X'X
     double d;                   // d value (largest eigenvalue of X'X)
-    double alpha;               // alpha = mixing parameter for elastic net
-    double gamma;               // extra tuning parameter for mcp/scad
     bool default_group_weights; // do we need to compute default group weights?
     int nfolds;                 // number of cross validation folds
     std::vector<MatrixXd > xtx_list;
@@ -72,13 +70,17 @@ protected:
     
     
     std::vector<std::vector<int> > grp_idx; // vector of vectors of the indexes for all members of each group
-    std::string penalty;       // penalty specified
+    std::string penalty;        // penalty specified
     
-    double lambda;             // L1 penalty
-    double lambda0;            // minimum lambda to make coefficients all zero
+    double lambda;              // L1 penalty
+    double lambda0;             // minimum lambda to make coefficients all zero
+    double alpha;               // alpha = mixing parameter for elastic net
+    double gamma;               // extra tuning parameter for mcp/scad
+    double tau;                 // mixing parameter for group sparse penalties
     
     double threshval;
     int wt_len;
+    bool found_grp_idx;
     
     static void soft_threshold(VectorXd &res, const VectorXd &vec, const double &penalty, 
                                VectorXd &pen_fact, double &d)
@@ -104,7 +106,7 @@ protected:
         int v_size = vec.size();
         res.setZero();
         double gammad = gamma * d;
-        double d_minus_gammainv = d - 1 / gamma;
+        double d_minus_gammainv = d - 1.0 / gamma;
         
         
         const double *ptr = vec.data();
@@ -129,7 +131,7 @@ protected:
         int v_size = vec.size();
         res.setZero();
         double gammad = gamma * d;
-        double gamma_minus1_d = (gamma - 1) * d;
+        double gamma_minus1_d = (gamma - 1.0) * d;
         
         const double *ptr = vec.data();
         for(int i = 0; i < v_size; i++)
@@ -138,14 +140,14 @@ protected:
             
             if (std::abs(ptr[i]) > gammad * total_pen)
                 res(i) = ptr[i]/d;
-            else if (std::abs(ptr[i]) > (d + 1) * total_pen)
+            else if (std::abs(ptr[i]) > (d + 1.0) * total_pen)
             {
-                double gam_ptr = (gamma - 1) * ptr[i];
+                double gam_ptr = (gamma - 1.0) * ptr[i];
                 double gam_pen = gamma * total_pen;
                 if(gam_ptr > gam_pen)
-                    res(i) = (gam_ptr - gam_pen)/(gamma_minus1_d - 1);
+                    res(i) = (gam_ptr - gam_pen)/(gamma_minus1_d - 1.0);
                 else if(gam_ptr < -gam_pen)
-                    res(i) = (gam_ptr + gam_pen)/(gamma_minus1_d - 1);
+                    res(i) = (gam_ptr + gam_pen)/(gamma_minus1_d - 1.0);
             }
             else if(ptr[i] > total_pen)
                 res(i) = (ptr[i] - total_pen)/d;
@@ -155,6 +157,132 @@ protected:
         }
     }
     
+    static double soft_threshold_scad_norm(double &b, const double &pen, double &d, double &gamma)
+    {
+        double retval = 0.0;
+        
+        double gammad = gamma * d;
+        double gamma_minus1_d = (gamma - 1.0) * d;
+        
+        if (std::abs(b) > gammad * pen)
+            retval = 1.0;
+        else if (std::abs(b) > (d + 1.0) * pen)
+        {
+            double gam_ptr = (gamma - 1.0);
+            double gam_pen = gamma * pen / b;
+            if(gam_ptr > gam_pen)
+                retval = d * (gam_ptr - gam_pen)/(gamma_minus1_d - 1.0);
+            else if(gam_ptr < -gam_pen)
+                retval = d * (gam_ptr + gam_pen)/(gamma_minus1_d - 1.0);
+        }
+        else if(b > pen)
+            retval = (1.0 - pen / b);
+        else if(b < -pen)
+            retval = (1.0 + pen / b);
+        return retval;
+    }
+    
+    static double soft_threshold_mcp_norm(double &b, const double &pen, double &d, double &gamma)
+    {
+        double retval = 0.0;
+        
+        double gammad = gamma * d;
+        double d_minus_gammainv = d - 1.0 / gamma;
+        
+        if (std::abs(b) > gammad * pen)
+            retval = 1.0;
+        else if(b > pen)
+            retval = d * (1.0 - pen / b)/(d_minus_gammainv);
+        else if(b < -pen)
+            retval = d * (1.0 + pen / b)/(d_minus_gammainv);
+        
+        return retval;
+    }
+    
+    static void block_soft_threshold_scad(VectorXd &res, const VectorXd &vec, const double &penalty,
+                                          VectorXd &pen_fact, double &d,
+                                          std::vector<std::vector<int> > &grp_idx, 
+                                          const int &ngroups, VectorXi &unique_grps, VectorXi &grps,
+                                          double & gamma)
+    {
+        //int v_size = vec.size();
+        res.setZero();
+        
+        for (int g = 0; g < ngroups; ++g) 
+        {
+            double thresh_factor;
+            std::vector<int> gr_idx = grp_idx[g];
+            
+            if (unique_grps(g) == 0) // the 0 group represents unpenalized variables
+            {
+                thresh_factor = 1.0;
+            } else 
+            {
+                double ds_norm = 0.0;
+                for (std::vector<int>::size_type v = 0; v < gr_idx.size(); ++v)
+                {
+                    int c_idx = gr_idx[v];
+                    ds_norm += std::pow(vec(c_idx), 2);
+                }
+                ds_norm = std::sqrt(ds_norm);
+                // double grp_wts = sqrt(gr_idx.size());
+                double grp_wts = pen_fact(g);
+                //thresh_factor = std::max(0.0, 1.0 - penalty * grp_wts / (ds_norm) );
+                thresh_factor = soft_threshold_scad_norm(ds_norm, penalty * grp_wts, d, gamma);
+            }
+            if (thresh_factor != 0.0)
+            {
+                for (std::vector<int>::size_type v = 0; v < gr_idx.size(); ++v)
+                {
+                    int c_idx = gr_idx[v];
+                    res(c_idx) = vec(c_idx) * thresh_factor / d;
+                }
+            }
+        }
+    }
+    
+    static void block_soft_threshold_mcp(VectorXd &res, const VectorXd &vec, const double &penalty,
+                                         VectorXd &pen_fact, double &d,
+                                         std::vector<std::vector<int> > &grp_idx, 
+                                         const int &ngroups, VectorXi &unique_grps, VectorXi &grps,
+                                         double & gamma)
+    {
+        //int v_size = vec.size();
+        res.setZero();
+        
+        for (int g = 0; g < ngroups; ++g) 
+        {
+            double thresh_factor;
+            std::vector<int> gr_idx = grp_idx[g];
+            
+            if (unique_grps(g) == 0) // the 0 group represents unpenalized variables
+            {
+                thresh_factor = 1.0;
+            } else 
+            {
+                double ds_norm = 0.0;
+                for (std::vector<int>::size_type v = 0; v < gr_idx.size(); ++v)
+                {
+                    int c_idx = gr_idx[v];
+                    ds_norm += std::pow(vec(c_idx), 2);
+                }
+                ds_norm = std::sqrt(ds_norm);
+                // double grp_wts = sqrt(gr_idx.size());
+                double grp_wts = pen_fact(g);
+                //thresh_factor = std::max(0.0, 1.0 - penalty * grp_wts / (ds_norm) );
+                thresh_factor = soft_threshold_mcp_norm(ds_norm, penalty * grp_wts, d, gamma);
+            }
+            if (thresh_factor != 0.0)
+            {
+                for (std::vector<int>::size_type v = 0; v < gr_idx.size(); ++v)
+                {
+                    int c_idx = gr_idx[v];
+                    res(c_idx) = vec(c_idx) * thresh_factor / d;
+                }
+            }
+        }
+    }
+    
     static void block_soft_threshold(VectorXd &res, const VectorXd &vec, const double &penalty,
                                      VectorXd &pen_fact, double &d,
                                      std::vector<std::vector<int> > &grp_idx, 
@@ -178,10 +306,10 @@ protected:
              */
             if (unique_grps(g) == 0) 
             {
-                thresh_factor = 1;
+                thresh_factor = 1.0;
             } else 
             {
-                double ds_norm = 0;
+                double ds_norm = 0.0;
                 for (std::vector<int>::size_type v = 0; v < gr_idx.size(); ++v)
                 {
                     int c_idx = gr_idx[v];
@@ -190,7 +318,7 @@ protected:
                 ds_norm = std::sqrt(ds_norm);
                 // double grp_wts = sqrt(gr_idx.size());
                 double grp_wts = pen_fact(g);
-                thresh_factor = std::max(0.0, 1 - penalty * grp_wts / (ds_norm) );
+                thresh_factor = std::max(0.0, 1.0 - penalty * grp_wts / (ds_norm) );
             }
             if (thresh_factor != 0.0)
             {
@@ -501,9 +629,11 @@ protected:
     
     void get_group_indexes()
     {
-        if (penalty == "grp.lasso") 
+        // if the group is any group penalty
+        std::string grptxt("grp");
+        if (penalty.find(grptxt) != std::string::npos)
         {
-            
+            found_grp_idx = true;
             grp_idx.reserve(ngroups);
             for (int g = 0; g < ngroups; ++g) 
             {
@@ -674,8 +804,8 @@ protected:
             }
         }
         
-        colsq /= (nobs - 1);
-        colsq_inv = 1 / colsq.array().sqrt();
+        colsq /= (double(nobs) - 1.0);
+        colsq_inv = 1.0 / colsq.array().sqrt();
         
         
         if (standardize)
@@ -740,21 +870,84 @@ protected:
             beta = u / d;
         } else if (penalty == "elastic.net")
         {
-            double denom = d + (1 - alpha) * lambda;
-            double lam = alpha * lambda;
+            double denom = d + (1.0 - alpha) * lambda / alpha;
+            double lam = lambda;
             soft_threshold(beta, u, lam, penalty_factor, denom);
         } else if (penalty == "scad") 
         {
             soft_threshold_scad(beta, u, lambda, penalty_factor, d, gamma);
+            
+        } else if (penalty == "scad.net") 
+        {
+            double denom = d + (1.0 - alpha) * lambda / alpha;
+            double lam = lambda;
+            soft_threshold_scad(beta, u, lam, penalty_factor, denom, gamma);
+            
         } else if (penalty == "mcp") 
         {
             soft_threshold_mcp(beta, u, lambda, penalty_factor, d, gamma);
+        } else if (penalty == "mcp.net") 
+        {
+            double denom = d + (1.0 - alpha) * lambda / alpha;
+            double lam = lambda;
+            soft_threshold_mcp(beta, u, lam, penalty_factor, denom, gamma);
+            
         } else if (penalty == "grp.lasso")
         {
             block_soft_threshold(beta, u, lambda, group_weights,
                                  d, grp_idx, ngroups, 
                                  unique_groups, groups);
-        }
+        } else if (penalty == "grp.lasso.net")
+        {
+            double denom = d + (1.0 - alpha) * lambda / alpha;
+            double lam = lambda;
+            block_soft_threshold(beta, u, lam, group_weights,
+                                 denom, grp_idx, ngroups, 
+                                 unique_groups, groups);
+        } else if (penalty == "grp.mcp")
+        {
+            block_soft_threshold_mcp(beta, u, lambda, group_weights,
+                                     d, grp_idx, ngroups, 
+                                     unique_groups, groups, gamma);
+        } else if (penalty == "grp.scad")
+        {
+            block_soft_threshold_scad(beta, u, lambda, group_weights,
+                                      d, grp_idx, ngroups, 
+                                      unique_groups, groups, gamma);
+        } else if (penalty == "grp.mcp.net")
+        {
+            double denom = d + (1.0 - alpha) * lambda / alpha;
+            double lam = lambda;
+            block_soft_threshold_mcp(beta, u, lam, group_weights,
+                                     denom, grp_idx, ngroups, 
+                                     unique_groups, groups, gamma);
+        } else if (penalty == "grp.scad.net")
+        {
+            double denom = d + (1.0 - alpha) * lambda / alpha;
+            double lam = lambda;
+            block_soft_threshold_scad(beta, u, lam, group_weights,
+                                      denom, grp_idx, ngroups, 
+                                      unique_groups, groups, gamma);
+        } else if (penalty == "sparse.grp.lasso")
+        {
+            double lam_grp = (1.0 - tau) * lambda;
+            double lam_l1  = tau * lambda;
+            
+            double fact = 1.0;
+            
+            // first apply soft thresholding
+            // but don't divide by d
+            soft_threshold(beta, u, lam_l1, penalty_factor, fact);
+            
+            VectorXd beta_tmp = beta;
+            
+            // then apply block soft thresholding
+            block_soft_threshold(beta, beta_tmp, lam_grp, 
+                                 group_weights,
+                                 d, grp_idx, ngroups, 
+                                 unique_groups, groups);
+        } 
+        
         
     }
     
@@ -769,8 +962,6 @@ public:
                  const VectorXi &unique_groups_,
                  VectorXd &group_weights_,
                  VectorXd &penalty_factor_,
-                 const double &alpha_,
-                 const double &gamma_,
                  bool &intercept_,
                  bool &standardize_,
                  const double tol_ = 1e-6) :
@@ -792,8 +983,6 @@ public:
                              XXdim( std::min(X_.cols(), X_.rows()) + intercept_ * (X_.rows() > X_.cols()) ),
                              XY(X_.cols() + intercept_),      // add extra space if intercept 
                              XX(XXdim, XXdim),                // add extra space if intercept 
-                             alpha(alpha_),
-                             gamma(gamma_),
                              default_group_weights(bool(group_weights_.size() < 1)), // compute default weights if none given
                              nfolds(nfolds_),
                              xtx_list(nfolds_),
@@ -813,6 +1002,8 @@ public:
         // compute XtX or XXt (depending on if n > p or not)
         // and compute A = dI - XtX (if n > p)
         
+        found_grp_idx = false;
+        
         compute_XtX_d_update_A(add_int_);
         
         if (intercept)
@@ -859,16 +1050,24 @@ public:
     double get_d() { return d; }
     
     // init() is a cold start for the first lambda
-    void init(double lambda_, std::string penalty_)
+    void init(double lambda_, std::string penalty_,
+              double alpha_, double gamma_, double tau_)
     {
         beta.setZero();
         
         lambda = lambda_;
         penalty = penalty_;
         
+        alpha = alpha_;
+        gamma = gamma_;
+        tau   = tau_;
+        
         // get indexes of members of each group.
         // best to do just once in the beginning
-        get_group_indexes();
+        if (!found_grp_idx)
+        {
+            get_group_indexes();
+        }
         
     }
     // when computing for the next lambda, we can use the
